@@ -64,6 +64,8 @@ enum ast_audiohook_flags {
 	AST_AUDIOHOOK_MUTE_READ     = (1 << 5), /*!< audiohook should be mute frames read */
 	AST_AUDIOHOOK_MUTE_WRITE    = (1 << 6), /*!< audiohook should be mute frames written */
 	AST_AUDIOHOOK_COMPATIBLE    = (1 << 7), /*!< is the audiohook native slin compatible */
+
+	AST_AUDIOHOOK_SUBSTITUTE_SILENCE = (1 << 8), /*!< Substitute silence for missing audio */
 };
 
 enum ast_audiohook_init_flags {
@@ -76,11 +78,12 @@ enum ast_audiohook_init_flags {
 struct ast_audiohook;
 
 /*! \brief Callback function for manipulate audiohook type
- * \param audiohook Audiohook structure
- * \param chan Channel
+ * \param audiohook
+ * \param chan
  * \param frame Frame of audio to manipulate
  * \param direction Direction frame came from
- * \return Returns 0 on success, -1 on failure.
+ * \retval 0 on success
+ * \retval -1 on failure
  * \note An audiohook does not have any reference to a private data structure for manipulate
  *       types. It is up to the manipulate callback to store this data via it's own method.
  *       An example would be datastores.
@@ -114,64 +117,69 @@ struct ast_audiohook {
 	struct ast_trans_pvt *trans_pvt;                       /*!< Translation path for reading frames */
 	ast_audiohook_manipulate_callback manipulate_callback; /*!< Manipulation callback */
 	struct ast_audiohook_options options;                  /*!< Applicable options */
-	unsigned int hook_internal_samp_rate;                           /*!< internal read/write sample rate on the audiohook.*/
+	unsigned int hook_internal_samp_rate;                  /*!< internal read/write sample rate on the audiohook.*/
 	AST_LIST_ENTRY(ast_audiohook) list;                    /*!< Linked list information */
 };
 
 struct ast_audiohook_list;
 
 /*! \brief Initialize an audiohook structure
- * \param audiohook Audiohook structure
+ * \param audiohook
  * \param type Type of audiohook to initialize this as
  * \param source Who is initializing this audiohook
- * \param init flags
- * \return Returns 0 on success, -1 on failure
+ * \param flags
+ * \retval 0 on success
+ * \retval -1 on failure
  */
 int ast_audiohook_init(struct ast_audiohook *audiohook, enum ast_audiohook_type type, const char *source, enum ast_audiohook_init_flags flags);
 
 /*! \brief Destroys an audiohook structure
- * \param audiohook Audiohook structure
- * \return Returns 0 on success, -1 on failure
+ * \param audiohook
+ * \retval 0 on success
+ * \retval -1 on failure
  */
 int ast_audiohook_destroy(struct ast_audiohook *audiohook);
 
 /*! \brief Writes a frame into the audiohook structure
- * \param audiohook Audiohook structure
+ * \param audiohook
  * \param direction Direction the audio frame came from
  * \param frame Frame to write in
- * \return Returns 0 on success, -1 on failure
+ * \retval 0 on success
+ * \retval -1 on failure
  */
 int ast_audiohook_write_frame(struct ast_audiohook *audiohook, enum ast_audiohook_direction direction, struct ast_frame *frame);
 
 /*! \brief Reads a frame in from the audiohook structure
- * \param audiohook Audiohook structure
+ * \param audiohook
  * \param samples Number of samples wanted
  * \param direction Direction the audio frame came from
  * \param format Format of frame remote side wants back
- * \return Returns frame on success, NULL on failure
+ * \return frame on success
+ * \retval NULL on failure
  */
 struct ast_frame *ast_audiohook_read_frame(struct ast_audiohook *audiohook, size_t samples, enum ast_audiohook_direction direction, struct ast_format *format);
 
 /*! \brief Reads a frame in from the audiohook structure in mixed audio mode and copies read and write frame data to provided arguments.
- * \param audiohook Audiohook structure
+ * \param audiohook
  * \param samples Number of samples wanted
- * \param ast_format Format of frame remote side wants back
+ * \param format Format of frame remote side wants back
  * \param read_frame if available, we'll copy the read buffer to this.
  * \param write_frame if available, we'll copy the write buffer to this.
- * \param direction
- * \return Returns frame on success, NULL on failure
+ * \return frame on success
+ * \retval NULL on failure
  */
 struct ast_frame *ast_audiohook_read_frame_all(struct ast_audiohook *audiohook, size_t samples, struct ast_format *format, struct ast_frame **read_frame, struct ast_frame **write_frame);
 
 /*! \brief Attach audiohook to channel
- * \param chan Channel
- * \param audiohook Audiohook structure
- * \return Returns 0 on success, -1 on failure
+ * \param chan
+ * \param audiohook
+ * \return 0 on success
+ * \retval -1 on failure
  */
 int ast_audiohook_attach(struct ast_channel *chan, struct ast_audiohook *audiohook);
 
 /*! \brief Detach audiohook from channel
- * \param audiohook Audiohook structure
+ * \param audiohook
  * \return Returns 0 on success, -1 on failure
  */
 int ast_audiohook_detach(struct ast_audiohook *audiohook);
@@ -179,7 +187,6 @@ int ast_audiohook_detach(struct ast_audiohook *audiohook);
 /*!
  * \brief Detach audiohooks from list and destroy said list
  * \param audiohook_list List of audiohooks (NULL tolerant)
- * \return Nothing
  */
 void ast_audiohook_detach_list(struct ast_audiohook_list *audiohook_list);
 
@@ -215,7 +222,8 @@ void ast_audiohook_move_all(struct ast_channel *old_chan, struct ast_channel *ne
  * \param chan Channel to detach from
  * \param source Name of source to detach
  *
- * \return Returns 0 on success, -1 on failure
+ * \retval 0 on success
+ * \retval -1 on failure
  *
  * \note The channel does not need to be locked before calling this function.
  */
@@ -227,7 +235,8 @@ int ast_audiohook_detach_source(struct ast_channel *chan, const char *source);
  * \param chan Channel to remove from
  * \param audiohook Audiohook to remove
  *
- * \return Returns 0 on success, -1 on failure
+ * \retval 0 on success
+ * \retval -1 on failure
  *
  * \note The channel does not need to be locked before calling this function
  */
@@ -236,9 +245,10 @@ int ast_audiohook_remove(struct ast_channel *chan, struct ast_audiohook *audioho
 /*!
  * \brief Determine if a audiohook_list is empty or not.
  *
- * \param audiohook Audiohook to check.  (NULL also means empty)
+ * \param audiohook_list Audiohook to check.  (NULL also means empty)
  *
- * retval 0 false, 1 true
+ * \retval 0 false
+ * \retval 1 true
  */
 int ast_audiohook_write_list_empty(struct ast_audiohook_list *audiohook_list);
 
@@ -247,13 +257,14 @@ int ast_audiohook_write_list_empty(struct ast_audiohook_list *audiohook_list);
  * \param audiohook_list List of audiohooks
  * \param direction Direction frame is coming in from
  * \param frame The frame itself
- * \return Return frame on success, NULL on failure
+ * \return frame on success
+ * \retval NULL on failure
  */
 struct ast_frame *ast_audiohook_write_list(struct ast_channel *chan, struct ast_audiohook_list *audiohook_list, enum ast_audiohook_direction direction, struct ast_frame *frame);
 
 /*! \brief Update audiohook's status
- * \param audiohook Audiohook structure
- * \param audiohook status enum
+ * \param audiohook
+ * \param status
  *
  * \note once status is updated to DONE, this function can not be used to set the
  * status back to any other setting.  Setting DONE effectively locks the status as such.
@@ -270,7 +281,7 @@ void ast_audiohook_trigger_wait(struct ast_audiohook *audiohook);
   \param chan The channel on which to find the spies
   \param source The audiohook's source
   \param type The type of audiohook
-  \return Return the number of audiohooks which are from the source specified
+  \return number of audiohooks which are from the source specified
 
   Note: Function performs nlocking.
 */
@@ -281,7 +292,7 @@ int ast_channel_audiohook_count_by_source(struct ast_channel *chan, const char *
   \param chan The channel on which to find the spies
   \param source The source of the audiohook
   \param type The type of spy to look for
-  \return Return the number of running audiohooks which are from the source specified
+  \return number of running audiohooks which are from the source specified
 
   Note: Function performs no locking.
 */
@@ -302,7 +313,8 @@ int ast_channel_audiohook_count_by_source_running(struct ast_channel *chan, cons
  * \param chan Channel to muck with
  * \param direction Direction to set on
  * \param volume Value to adjust the volume by
- * \return Returns 0 on success, -1 on failure
+ * \retval 0 on success
+ * \retval -1 on failure
  * \since 1.6.1
  */
 int ast_audiohook_volume_set(struct ast_channel *chan, enum ast_audiohook_direction direction, int volume);
@@ -311,7 +323,7 @@ int ast_audiohook_volume_set(struct ast_channel *chan, enum ast_audiohook_direct
  * \brief Retrieve the volume adjustment value on frames read from or written to a channel
  * \param chan Channel to retrieve volume adjustment from
  * \param direction Direction to retrieve
- * \return Returns adjustment value
+ * \return adjustment value
  * \since 1.6.1
  */
 int ast_audiohook_volume_get(struct ast_channel *chan, enum ast_audiohook_direction direction);
@@ -321,7 +333,8 @@ int ast_audiohook_volume_get(struct ast_channel *chan, enum ast_audiohook_direct
  * \param chan Channel to muck with
  * \param direction Direction to increase
  * \param volume Value to adjust the adjustment by
- * \return Returns 0 on success, -1 on failure
+ * \retval 0 on success
+ * \retval -1 on failure
  * \since 1.6.1
  */
 int ast_audiohook_volume_adjust(struct ast_channel *chan, enum ast_audiohook_direction direction, int volume);
