@@ -109,7 +109,7 @@ struct ast_state_cb_info {
 };
 
 /*! \brief Typedef for devicestate and hint callbacks */
-typedef int (*ast_state_cb_type)(char *context, char *id, struct ast_state_cb_info *info, void *data);
+typedef int (*ast_state_cb_type)(const char *context, const char *exten, struct ast_state_cb_info *info, void *data);
 
 /*! \brief Typedef for devicestate and hint callback removal indication callback */
 typedef void (*ast_state_cb_destroy_type)(int id, void *data);
@@ -144,7 +144,7 @@ struct ast_custom_function {
 					 * 'dangerous', and should not be run directly
 					 * from external interfaces (AMI, ARI, etc.)
 					 * \since 12 */
-	unsigned int write_escalates:1; /*!< The write function is to be considerd
+	unsigned int write_escalates:1; /*!< The write function is to be considered
 					 * 'dangerous', and should not be run directly
 					 * from external interfaces (AMI, ARI, etc.)
 					 * \since 12 */
@@ -180,15 +180,17 @@ struct ast_timing {
 /*!
  * \brief Construct a timing bitmap, for use in time-based conditionals.
  * \param i Pointer to an ast_timing structure.
- * \param info Standard string containing a timerange, weekday range, monthday range, and month range, as well as an optional timezone.
- * \retval Returns 1 on success or 0 on failure.
+ * \param info_in Standard string containing a timerange, weekday range, monthday range, and month range, as well as an optional timezone.
+ * \retval 1 on success.
+ * \retval 0 on failure.
  */
-int ast_build_timing(struct ast_timing *i, const char *info);
+int ast_build_timing(struct ast_timing *i, const char *info_in);
 
 /*!
  * \brief Evaluate a pre-constructed bitmap as to whether the current time falls within the range specified.
  * \param i Pointer to an ast_timing structure.
- * \retval Returns 1, if the time matches or 0, if the current time falls outside of the specified range.
+ * \retval 1 if the time matches.
+ * \retval 0 if the current time falls outside of the specified range.
  */
 int ast_check_timing(const struct ast_timing *i);
 
@@ -196,7 +198,8 @@ int ast_check_timing(const struct ast_timing *i);
  * \brief Evaluate a pre-constructed bitmap as to whether a particular time falls within the range specified.
  * \param i Pointer to an ast_timing structure.
  * \param tv Specified time
- * \retval Returns 1, if the time matches or 0, if the time falls outside of the specified range.
+ * \retval 1 if the time matches.
+ * \retval 0 if the time falls outside of the specified range.
  */
 int ast_check_timing2(const struct ast_timing *i, const struct timeval tv);
 
@@ -233,8 +236,6 @@ int ast_register_switch(struct ast_switch *sw);
  * \param sw switch to unregister
  *
  * Unregisters a switch from asterisk.
- *
- * \return nothing
  */
 void ast_unregister_switch(struct ast_switch *sw);
 
@@ -287,6 +288,15 @@ int pbx_exec(struct ast_channel *c, struct ast_app *app, const char *data);
  */
 struct ast_context *ast_context_find_or_create(struct ast_context **extcontexts, struct ast_hashtab *exttable, const char *name, const char *registrar);
 
+/*!
+ * \brief Enable or disable autohints support on a context
+ *
+ * \param con pointer to the context
+ * \param enabled whether autohints are enabled
+ *
+ */
+void ast_context_set_autohints(struct ast_context *con, int enabled);
+
 /*!
  * \brief Merge the temporary contexts into a global contexts list and delete from the
  *        global list the ones that are being added
@@ -321,8 +331,6 @@ int ast_context_destroy_by_name(const char *context, const char *registrar);
  *
  * You can optionally leave out either parameter.  It will find it
  * based on either the ast_context or the registrar name.
- *
- * \return nothing
  */
 void ast_context_destroy(struct ast_context *con, const char *registrar);
 
@@ -413,8 +421,6 @@ enum ast_pbx_result ast_pbx_run_args(struct ast_channel *c, struct ast_pbx_args
  *
  * \param chan Channel to run the h exten on.
  * \param context Context the h exten is in.
- *
- * \return Nothing
  */
 void ast_pbx_h_exten_run(struct ast_channel *chan, const char *context);
 
@@ -436,8 +442,6 @@ int ast_pbx_hangup_handler_run(struct ast_channel *chan);
  * \since 11.0
  *
  * \param chan Channel to init the hangup handler container on.
- *
- * \return Nothing
  */
 void ast_pbx_hangup_handler_init(struct ast_channel *chan);
 
@@ -446,8 +450,6 @@ void ast_pbx_hangup_handler_init(struct ast_channel *chan);
  * \since 11.0
  *
  * \param chan Channel to destroy the hangup handler container on.
- *
- * \return Nothing
  */
 void ast_pbx_hangup_handler_destroy(struct ast_channel *chan);
 
@@ -467,8 +469,6 @@ int ast_pbx_hangup_handler_pop(struct ast_channel *chan);
  *
  * \param chan Channel to push the hangup handler onto.
  * \param handler Gosub application parameter string.
- *
- * \return Nothing
  */
 void ast_pbx_hangup_handler_push(struct ast_channel *chan, const char *handler);
 
@@ -483,9 +483,13 @@ void ast_pbx_hangup_handler_push(struct ast_channel *chan, const char *handler);
  * \param callerid pattern to match CallerID, or NULL to match any CallerID
  * \param application application to run on the extension with that priority level
  * \param data data to pass to the application
- * \param datad
+ * \param datad a pointer to a function that will deallocate \c data when needed
+ *              or NULL if \c data does not need to be freed.
  * \param registrar who registered the extension
  *
+ * \note On any failure, the function pointed to by \c datap will be called and passed the
+ *       \c data pointer.
+ *
  * \retval 0 success
  * \retval -1 failure
  */
@@ -496,22 +500,42 @@ int ast_add_extension(const char *context, int replace, const char *extension,
 /*!
  * \brief Add an extension to an extension context, this time with an ast_context *.
  *
- * \note For details about the arguments, check ast_add_extension()
+ * \param con context to add the extension to
+ * \param replace
+ * \param extension extension to add
+ * \param priority priority level of extension addition
+ * \param label extension label
+ * \param callerid pattern to match CallerID, or NULL to match any CallerID
+ * \param application application to run on the extension with that priority level
+ * \param data data to pass to the application
+ * \param datad a pointer to a function that will deallocate \c data when needed
+ *              or NULL if \c data does not need to be freed.
+ * \param registrar who registered the extension
+ * \param registrar_file optional configuration file that defines this extension
+ * \param registrar_line optional line number of configuration file that defines extension
+ *
+ * \note On any failure, the function pointed to by \c datap will be called and passed the
+ *       \c data pointer.
+ *
+ * \retval 0 success
+ * \retval -1 failure
  */
 int ast_add_extension2(struct ast_context *con, int replace, const char *extension,
 	int priority, const char *label, const char *callerid,
-	const char *application, void *data, void (*datad)(void *), const char *registrar);
+	const char *application, void *data, void (*datad)(void *), const char *registrar,
+	const char *registrar_file, int registrar_line);
 
 /*!
  * \brief Same as ast_add_extension2, but assumes you have already locked context
  * \since 12.0.0
  *
  * \note con must be write locked prior to calling. For details about the arguments,
- *       check ast_add_extension2()
+ *       check ast_add_extension()
  */
 int ast_add_extension2_nolock(struct ast_context *con, int replace, const char *extension,
 	int priority, const char *label, const char *callerid,
-	const char *application, void *data, void (*datad)(void *), const char *registrar);
+	const char *application, void *data, void (*datad)(void *), const char *registrar,
+	const char *registrar_file, int registrar_line);
 
 /*!
  * \brief Map devstate to an extension state.
@@ -771,7 +795,7 @@ int ast_findlabel_extension2(struct ast_channel *c, struct ast_context *con,
  * \brief Looks for a valid matching extension
  *
  * \param c not really important
- * \param context context to serach within
+ * \param context context to search within
  * \param exten extension to check
  * \param priority priority of extension path
  * \param callerid callerid of extension being searched for
@@ -791,7 +815,7 @@ int ast_canmatch_extension(struct ast_channel *c, const char *context,
  * \brief Looks to see if adding anything to this extension might match something. (exists ^ canmatch)
  *
  * \param c not really important XXX
- * \param context context to serach within
+ * \param context context to search within
  * \param exten extension to check
  * \param priority priority of extension path
  * \param callerid callerid of extension being searched for
@@ -878,25 +902,17 @@ int ast_context_add_include(const char *context, const char *include,
 /*!
  * \brief Add a context include
  *
- * \param con context to add the include to
- * \param value include value to add
- * \param registrar who registered the context
- *
  * Adds an include taking a struct ast_context as the first parameter
  *
- * \retval 0 on success
- * \retval -1 on failure
+ * \note See ast_context_add_include for information on arguments
  */
-int ast_context_add_include2(struct ast_context *con, const char *include,
+int ast_context_add_include2(struct ast_context *con, const char *value,
 	const char *registrar);
 
 /*!
  * \brief Remove a context include
  *
  * \note See ast_context_add_include for information on arguments
- *
- * \retval 0 on success
- * \retval -1 on failure
  */
 int ast_context_remove_include(const char *context, const char *include,
 	const char *registrar);
@@ -904,10 +920,7 @@ int ast_context_remove_include(const char *context, const char *include,
 /*!
  * \brief Removes an include by an ast_context structure
  *
- * \note See ast_context_add_include2 for information on arguments
- *
- * \retval 0 on success
- * \retval -1 on success
+ * \note See ast_context_add_include for information on arguments
  */
 int ast_context_remove_include2(struct ast_context *con, const char *include,
 	const char *registrar);
@@ -995,7 +1008,7 @@ int ast_context_remove_extension_callerid2(struct ast_context *con, const char *
 /*!
  * \brief Add an ignorepat
  *
- * \param context which context to add the ignorpattern to
+ * \param context which context to add the ignorepattern to
  * \param ignorepat ignorepattern to set up for the extension
  * \param registrar registrar of the ignore pattern
  *
@@ -1008,7 +1021,7 @@ int ast_context_add_ignorepat(const char *context, const char *ignorepat, const
 
 int ast_context_add_ignorepat2(struct ast_context *con, const char *ignorepat, const char *registrar);
 
-/*
+/*!
  * \brief Remove an ignorepat
  *
  * \param context context from which to remove the pattern
@@ -1156,8 +1169,8 @@ enum ast_pbx_outgoing_sync {
  *        If \c AST_OUTGOING_WAIT_COMPLETE then wait for the call to complete or
  *        fail.
  *        If \c AST_OUTGOING_WAIT or \c AST_OUTGOING_WAIT_COMPLETE is specified,
- *        the call doesn't answer, and \c failed@context exists then run a channel
- *        named \c OutgoingSpoolFailed at \c failed@context.
+ *        the call doesn't answer, and \c failed\@context exists then run a channel
+ *        named \c OutgoingSpoolFailed at \c failed\@context.
  * \param cid_num The caller ID number to set on the outbound channel
  * \param cid_name The caller ID name to set on the outbound channel
  * \param vars Variables to set on the outbound channel
@@ -1177,6 +1190,12 @@ int ast_pbx_outgoing_exten(const char *type, struct ast_format_cap *cap, const c
 	const char *account, struct ast_channel **locked_channel, int early_media,
 	const struct ast_assigned_ids *assignedids);
 
+int ast_pbx_outgoing_exten_predial(const char *type, struct ast_format_cap *cap, const char *addr,
+	int timeout, const char *context, const char *exten, int priority, int *reason,
+	int synchronous, const char *cid_num, const char *cid_name, struct ast_variable *vars,
+	const char *account, struct ast_channel **locked_channel, int early_media,
+	const struct ast_assigned_ids *assignedids, const char *predial_callee);
+
 /*!
  * \brief Synchronously or asynchronously make an outbound call and execute an
  *  application on the channel.
@@ -1215,6 +1234,12 @@ int ast_pbx_outgoing_app(const char *type, struct ast_format_cap *cap, const cha
 	const char *account, struct ast_channel **locked_channel,
 	const struct ast_assigned_ids *assignedids);
 
+int ast_pbx_outgoing_app_predial(const char *type, struct ast_format_cap *cap, const char *addr,
+	int timeout, const char *app, const char *appdata, int *reason, int synchronous,
+	const char *cid_num, const char *cid_name, struct ast_variable *vars,
+	const char *account, struct ast_channel **locked_channel,
+	const struct ast_assigned_ids *assignedids, const char *predial_callee);
+
 /*!
  * \brief Evaluate a condition
  *
@@ -1231,11 +1256,11 @@ int pbx_checkcondition(const char *condition);
 const char *ast_get_context_name(struct ast_context *con);
 const char *ast_get_extension_name(struct ast_exten *exten);
 struct ast_context *ast_get_extension_context(struct ast_exten *exten);
-const char *ast_get_include_name(struct ast_include *include);
-const char *ast_get_ignorepat_name(struct ast_ignorepat *ip);
-const char *ast_get_switch_name(struct ast_sw *sw);
-const char *ast_get_switch_data(struct ast_sw *sw);
-int ast_get_switch_eval(struct ast_sw *sw);
+const char *ast_get_include_name(const struct ast_include *include);
+const char *ast_get_ignorepat_name(const struct ast_ignorepat *ip);
+const char *ast_get_switch_name(const struct ast_sw *sw);
+const char *ast_get_switch_data(const struct ast_sw *sw);
+int ast_get_switch_eval(const struct ast_sw *sw);
 
 /*! @} */
 
@@ -1247,17 +1272,50 @@ const char *ast_get_extension_cidmatch(struct ast_exten *e);
 const char *ast_get_extension_app(struct ast_exten *e);
 const char *ast_get_extension_label(struct ast_exten *e);
 void *ast_get_extension_app_data(struct ast_exten *e);
+
+/*!
+ * \brief Fill a string buffer with the data at a dialplan extension
+ *
+ * \param buf String buffer
+ * \param bufsize Size of buf
+ * \param c Channel
+ * \param context Dialplan context
+ * \param exten Dialplan extension
+ * \param priority Dialplan priority
+ *
+ * \retval -1 Failed to obtain extension data
+ * \retval 0 Successfully obtained extension data
+ */
+int ast_get_extension_data(char *buf, int bufsize, struct ast_channel *c,
+	const char *context, const char *exten, int priority);
 /*! @} */
 
 /*! @name Registrar info functions ... */
 /*! @{ */
 const char *ast_get_context_registrar(struct ast_context *c);
 const char *ast_get_extension_registrar(struct ast_exten *e);
-const char *ast_get_include_registrar(struct ast_include *i);
-const char *ast_get_ignorepat_registrar(struct ast_ignorepat *ip);
-const char *ast_get_switch_registrar(struct ast_sw *sw);
+const char *ast_get_include_registrar(const struct ast_include *i);
+const char *ast_get_ignorepat_registrar(const struct ast_ignorepat *ip);
+const char *ast_get_switch_registrar(const struct ast_sw *sw);
 /*! @} */
 
+/*!
+ * \brief Get name of configuration file used by registrar to register this extension
+ *
+ * \retval NULL if registrar did not indicate config file when registering the extension
+ * \retval name of the file used to register the extension
+ */
+const char *ast_get_extension_registrar_file(struct ast_exten *e);
+
+/*!
+ * \brief Get line number of configuration file used by registrar to register this extension
+ *
+ * \retval 0 if the line wasn't indicated when the extension was registered
+ * \retval positive integer indicating what line in the config file was responsible for
+ *         registering the extension.
+ */
+int ast_get_extension_registrar_line(struct ast_exten *e);
+
 /*! @name Walking functions ... */
 /*! @{ */
 struct ast_context *ast_walk_contexts(struct ast_context *con);
@@ -1265,11 +1323,22 @@ struct ast_exten *ast_walk_context_extensions(struct ast_context *con,
 	struct ast_exten *priority);
 struct ast_exten *ast_walk_extension_priorities(struct ast_exten *exten,
 	struct ast_exten *priority);
-struct ast_include *ast_walk_context_includes(struct ast_context *con,
-	struct ast_include *inc);
-struct ast_ignorepat *ast_walk_context_ignorepats(struct ast_context *con,
-	struct ast_ignorepat *ip);
-struct ast_sw *ast_walk_context_switches(struct ast_context *con, struct ast_sw *sw);
+const struct ast_include *ast_walk_context_includes(const struct ast_context *con,
+	const struct ast_include *inc);
+const struct ast_ignorepat *ast_walk_context_ignorepats(const struct ast_context *con,
+	const struct ast_ignorepat *ip);
+const struct ast_sw *ast_walk_context_switches(const struct ast_context *con,
+	const struct ast_sw *sw);
+/*! @} */
+
+/*! @name Iterator functions ... */
+/*! @{ */
+int ast_context_includes_count(const struct ast_context *con);
+const struct ast_include *ast_context_includes_get(const struct ast_context *con, int idx);
+int ast_context_ignorepats_count(const struct ast_context *con);
+const struct ast_ignorepat *ast_context_ignorepats_get(const struct ast_context *con, int idx);
+int ast_context_switches_count(const struct ast_context *con);
+const struct ast_sw *ast_context_switches_get(const struct ast_context *con, int idx);
 /*! @} */
 
 /*!
@@ -1342,10 +1411,15 @@ int pbx_builtin_raise_exception(struct ast_channel *chan, const char *data);
 void pbx_substitute_variables_helper(struct ast_channel *c, const char *cp1, char *cp2, int count);
 void pbx_substitute_variables_varshead(struct varshead *headp, const char *cp1, char *cp2, int count);
 void pbx_substitute_variables_helper_full(struct ast_channel *c, struct varshead *headp, const char *cp1, char *cp2, int cp2_size, size_t *used);
-/*! @} */
+
+/*!
+ * \brief Substitutes variables, similar to pbx_substitute_variables_helper_full, but allows passing the context, extension, and priority in.
+ */
+void pbx_substitute_variables_helper_full_location(struct ast_channel *c, struct varshead *headp, const char *cp1, char *cp2, int cp2_size, size_t *used, char *context, char *exten, int pri);
 /*! @} */
 
-/*! @name Substitution routines, using dynamic string buffers */
+/*! @name Substitution routines, using dynamic string buffers
+ * @{ */
 
 /*!
  * \param buf Result will be placed in this buffer.
@@ -1447,7 +1521,7 @@ enum ast_custom_function_escalation {
 /*!
  * \brief Register a custom function
  */
-#define ast_custom_function_register(acf) __ast_custom_function_register(acf, ast_module_info->self)
+#define ast_custom_function_register(acf) __ast_custom_function_register(acf, AST_MODULE_SELF)
 
 /*!
  * \brief Register a custom function which requires escalated privileges.
@@ -1456,7 +1530,7 @@ enum ast_custom_function_escalation {
  * arbitrary code) or FILE() (for which write needs permission to change files
  * on the filesystem).
  */
-#define ast_custom_function_register_escalating(acf, escalation) __ast_custom_function_register_escalating(acf, escalation, ast_module_info->self)
+#define ast_custom_function_register_escalating(acf, escalation) __ast_custom_function_register_escalating(acf, escalation, AST_MODULE_SELF)
 
 /*!
  * \brief Register a custom function
@@ -1548,7 +1622,12 @@ enum ext_match_t {
 #define STATUS_NO_PRIORITY	3
 #define STATUS_NO_LABEL		4
 #define STATUS_SUCCESS		5
+
+#ifdef LOW_MEMORY
 #define AST_PBX_MAX_STACK  128
+#else
+#define AST_PBX_MAX_STACK  512
+#endif
 
 /* request and result for pbx_find_extension */
 struct pbx_find_info {
